@@ -11,6 +11,7 @@
 #include "alloc/accounting.h"
 #include "alloc/background.h"
 #include "alloc/buckets.h"
+#include "alloc/discard.h"
 #include "alloc/disk_groups.h"
 #include "alloc/foreground.h"
 #include "alloc/replicas.h"
@@ -27,10 +28,10 @@
 
 #include "data/compress.h"
 #include "data/copygc.h"
-#include "data/ec.h"
+#include "data/ec/create.h"
 #include "data/move.h"
 #include "data/nocow_locking.h"
-#include "data/reconcile.h"
+#include "data/reconcile/work.h"
 
 #include "debug/sysfs.h"
 #include "debug/tests.h"
@@ -182,6 +183,8 @@ read_attribute(io_latency_read);
 read_attribute(io_latency_write);
 read_attribute(io_latency_stats_read);
 read_attribute(io_latency_stats_write);
+read_attribute(io_latency_stats_read_json);
+read_attribute(io_latency_stats_write_json);
 #ifndef CONFIG_BCACHEFS_NO_LATENCY_ACCT
 read_attribute(congested);
 #endif
@@ -195,6 +198,7 @@ read_attribute(journal_debug);
 read_attribute(btree_cache);
 read_attribute(btree_key_cache);
 read_attribute(btree_reserve_cache);
+read_attribute(btree_write_buffer);
 read_attribute(open_buckets);
 read_attribute(open_buckets_partial);
 read_attribute(nocow_lock_table);
@@ -364,6 +368,9 @@ SHOW(bch2_fs)
 	if (attr == &sysfs_btree_reserve_cache)
 		bch2_btree_reserve_cache_to_text(out, c);
 
+	if (attr == &sysfs_btree_write_buffer)
+		bch2_btree_write_buffer_to_text(out, c);
+
 	if (attr == &sysfs_open_buckets)
 		bch2_open_buckets_to_text(out, c, NULL);
 
@@ -610,6 +617,7 @@ struct attribute *bch2_fs_internal_files[] = {
 	&sysfs_btree_cache,
 	&sysfs_btree_key_cache,
 	&sysfs_btree_reserve_cache,
+	&sysfs_btree_write_buffer,
 	&sysfs_new_stripes,
 	&sysfs_open_buckets,
 	&sysfs_open_buckets_partial,
@@ -651,6 +659,96 @@ struct attribute *bch2_fs_internal_files[] = {
 	NULL
 };
 
+/* btree transaction stats - JSON via bin_attribute */
+
+static ssize_t bch2_btree_trans_stats_json_read(struct file *file,
+		struct kobject *kobj, const struct bin_attribute *attr,
+		char *buf, loff_t off, size_t count)
+{
+	struct bch_fs *c = container_of(kobj, struct bch_fs, internal);
+	struct printbuf *out = &c->btree.trans.stats_json_buf;
+
+	guard(mutex)(&c->btree.trans.stats_json_lock);
+
+	/*
+	 * kernfs caps bin_attribute reads at PAGE_SIZE per callback, so
+	 * multi-page output triggers multiple calls with increasing offsets.
+	 * Regenerating each time would produce inconsistent JSON if stats
+	 * change between calls. Cache the buffer and only regenerate when
+	 * off == 0 (start of a new read sequence).
+	 */
+	if (off == 0) {
+		printbuf_reset(out);
+
+		bool first = true;
+
+		prt_char(out, '{');
+
+		for (unsigned i = 0; i < ARRAY_SIZE(bch2_btree_transaction_fns); i++) {
+			if (!bch2_btree_transaction_fns[i])
+				break;
+
+			struct btree_transaction_stats *s = &c->btree.trans.stats[i];
+
+			if (!first)
+				prt_char(out, ',');
+			first = false;
+
+			prt_printf(out, "\"%s\":{", bch2_btree_transaction_fns[i]);
+
+			prt_printf(out, "\"max_mem\":%u,", s->max_mem);
+
+			prt_str(out, "\"duration\":");
+			bch2_time_stats_json_to_text(out, &s->duration, NULL, 0);
+
+			if (IS_ENABLED(CONFIG_BCACHEFS_LOCK_TIME_STATS)) {
+				prt_str(out, ",\"lock_hold_times\":");
+				bch2_time_stats_json_to_text(out, &s->lock_hold_times, NULL, 0);
+			}
+
+			prt_char(out, '}');
+		}
+
+		prt_str(out, "}\n");
+
+		if (out->allocation_failure)
+			return -ENOMEM;
+	}
+
+	if (off >= out->pos)
+		return 0;
+
+	size_t n = min_t(size_t, count, out->pos - off);
+	memcpy(buf, out->buf + off, n);
+	return n;
+}
+
+static ssize_t bch2_btree_trans_stats_json_write(struct file *file,
+		struct kobject *kobj, const struct bin_attribute *attr,
+		char *buf, loff_t off, size_t count)
+{
+	struct bch_fs *c = container_of(kobj, struct bch_fs, internal);
+
+	for (unsigned i = 0; i < ARRAY_SIZE(bch2_btree_transaction_fns); i++) {
+		if (!bch2_btree_transaction_fns[i])
+			break;
+
+		struct btree_transaction_stats *s = &c->btree.trans.stats[i];
+
+		guard(mutex)(&s->lock);
+		bch2_time_stats_reset(&s->duration);
+		bch2_time_stats_reset(&s->lock_hold_times);
+	}
+
+	return count;
+}
+
+struct bin_attribute bin_attr_btree_trans_stats_json = {
+	.attr	= { .name = "btree_trans_stats_json", .mode = 0644 },
+	.read	= bch2_btree_trans_stats_json_read,
+	.write	= bch2_btree_trans_stats_json_write,
+};
+
 /* options */
 
 static ssize_t sysfs_opt_show(struct bch_fs *c,
@@ -682,6 +780,10 @@ static ssize_t sysfs_opt_store(struct bch_fs *c,
 	const struct bch_option *opt = bch2_opt_table + id;
 	int ret = 0;
 
+	char *tmp __free(kfree) = kstrdup(buf, GFP_KERNEL);
+	if (!tmp)
+		return -ENOMEM;
+
 	/*
 	 * We don't need to take c->writes for correctness, but it eliminates an
 	 * unsightly error message in the dmesg log when we're RO:
@@ -689,43 +791,39 @@ static ssize_t sysfs_opt_store(struct bch_fs *c,
 	if (unlikely(!enumerated_ref_tryget(&c->writes, BCH_WRITE_REF_sysfs)))
 		return -EROFS;
 
-	char *tmp __free(kfree) = kstrdup(buf, GFP_KERNEL);
-	if (!tmp) {
-		ret = -ENOMEM;
-		goto err;
-	}
+	guard(memalloc_flags)(PF_MEMALLOC_NOFS);
+	guard(opt_change_lock)(c);
 
 	u64 v;
 	ret =   bch2_opt_parse(c, opt, strim(tmp), &v, NULL) ?:
 		bch2_opt_hook_pre_set(c, ca, 0, id, v, true);
 
-	if (ret < 0)
-		goto err;
+	if (!ret) {
+		bool is_sb = opt->get_sb || opt->get_member;
+		bool changed = false;
 
-	bool is_sb = opt->get_sb || opt->get_member;
-	bool changed = false;
-
-	if (is_sb) {
-		changed = bch2_opt_set_sb(c, ca, opt, v);
-	} else if (!ca) {
-		changed = bch2_opt_get_by_id(&c->opts, id) != v;
-	} else {
-		/* device options that aren't superblock options aren't
-		 * supported */
-		BUG();
-	}
-
-	if (changed) {
-		if (!ca) {
-			bch2_opt_set_by_id(&c->opts, id, v);
-			clear_bit(id, c->mount_opts.d);
+		if (is_sb) {
+			changed = bch2_opt_set_sb(c, ca, opt, v);
+		} else if (!ca) {
+			changed = bch2_opt_get_by_id(&c->opts, id) != v;
+		} else {
+			/* device options that aren't superblock options aren't
+			 * supported */
+			BUG();
 		}
 
-		bch2_opt_hook_post_set(c, ca, 0, id, v);
+		if (changed) {
+			if (!ca) {
+				bch2_opt_set_by_id(&c->opts, id, v);
+				clear_bit(id, c->mount_opts.d);
+			}
+
+			bch2_opt_hook_post_set(c, ca, 0, id, v);
+		}
+
+		ret = size;
 	}
 
-	ret = size;
-err:
 	enumerated_ref_put(&c->writes, BCH_WRITE_REF_sysfs);
 	return ret;
 }
@@ -805,6 +903,42 @@ struct attribute *bch2_fs_time_stats_files[] = {
 	NULL
 };
 
+/* time stats - JSON */
+
+SHOW(bch2_fs_time_stats_json)
+{
+	struct bch_fs *c = container_of(kobj, struct bch_fs, time_stats_json);
+
+#define x(name)								\
+	if (attr == &sysfs_time_stat_##name)				\
+		bch2_time_stats_json_to_text(out, &c->times[BCH_TIME_##name], NULL, 0);
+	BCH_TIME_STATS()
+#undef x
+
+	return 0;
+}
+
+STORE(bch2_fs_time_stats_json)
+{
+	struct bch_fs *c = container_of(kobj, struct bch_fs, time_stats_json);
+
+#define x(name)								\
+	if (attr == &sysfs_time_stat_##name)				\
+		bch2_time_stats_reset(&c->times[BCH_TIME_##name]);
+	BCH_TIME_STATS()
+#undef x
+	return size;
+}
+SYSFS_OPS(bch2_fs_time_stats_json);
+
+struct attribute *bch2_fs_time_stats_json_files[] = {
+#define x(name)						\
+	&sysfs_time_stat_##name,
+	BCH_TIME_STATS()
+#undef x
+	NULL
+};
+
 static const char * const bch2_rw[] = {
 	"read",
 	"write",
@@ -861,6 +995,12 @@ SHOW(bch2_dev)
 	if (attr == &sysfs_io_latency_stats_write)
 		bch2_time_stats_to_text(out, &ca->io_latency[WRITE].stats);
 
+	if (attr == &sysfs_io_latency_stats_read_json)
+		bch2_time_stats_json_to_text(out, &ca->io_latency[READ].stats, NULL, 0);
+
+	if (attr == &sysfs_io_latency_stats_write_json)
+		bch2_time_stats_json_to_text(out, &ca->io_latency[WRITE].stats, NULL, 0);
+
 #ifndef CONFIG_BCACHEFS_NO_LATENCY_ACCT
 	if (attr == &sysfs_congested)
 		bch2_dev_congested_to_text(out, ca);
@@ -926,6 +1066,8 @@ struct attribute *bch2_dev_files[] = {
 	&sysfs_io_latency_write,
 	&sysfs_io_latency_stats_read,
 	&sysfs_io_latency_stats_write,
+	&sysfs_io_latency_stats_read_json,
+	&sysfs_io_latency_stats_write_json,
 #ifndef CONFIG_BCACHEFS_NO_LATENCY_ACCT
 	&sysfs_congested,
 #endif
