@@ -22,10 +22,11 @@ fn list_keys(fs: &Fs, opt: &Cli) -> anyhow::Result<()> {
         flags |= BtreeIterFlags::ALL_SNAPSHOTS;
     }
 
-    let mut iter = BtreeIter::new(
+    let mut iter = BtreeIter::new_level(
         &trans,
         opt.btree,
         opt.start,
+        opt.level,
         flags,
     );
 
@@ -195,6 +196,7 @@ fn cmd_list_inner(opt: &Cli) -> anyhow::Result<()> {
         opt_set!(fs_opts, verbose, 1);
     }
 
+    // scan for member devices
     let fs = Fs::open(&opt.devices, fs_opts)?;
 
     match opt.mode {
